import { Module } from '@nestjs/common';
import { MongooseModule } from '@nestjs/mongoose';
import { ScheduleModule } from '@nestjs/schedule';
import { AdminModule } from './admin/admin.module';
import { AppController } from './app.controller';
import { AuthModule } from './auth/auth.module';
import { CategoriesModule } from './categories/categories.module';
import { ConfigurationModule } from './configuration/configuration.module';
import { ContactModule } from './contact/contact.module';
import { MailerModule } from './mailer/mailer.module';
import { NewsletterModule } from './newsletter/newsletter.module';
import { PagesModule } from './pages/pages.module';
import { ParametersModule } from './parameters/parameters.module';
import { PersonalOffersModule } from './personal-offers/personal-offers.module';
import { PostsModule } from './posts/posts.module';
import { StructuresModule } from './structures/structures.module';
import { TclModule } from './tcl/tcl.module';
import { TempUserModule } from './temp-user/temp-user.module';
import { UsersModule } from './users/users.module';
@Module({
  imports: [
    ConfigurationModule,
    MongooseModule.forRoot(
      `mongodb://${process.env.MONGO_NON_ROOT_USERNAME}:${process.env.MONGO_NON_ROOT_PASSWORD}@${process.env.MONGO_DB_HOST_AND_PORT}/ram`
    ),
    ScheduleModule.forRoot(),
    StructuresModule,
    CategoriesModule,
    AuthModule,
    UsersModule,
    MailerModule,
    TclModule,
    AdminModule,
    PostsModule,
    PagesModule,
    TempUserModule,
    NewsletterModule,
    ContactModule,
    PersonalOffersModule,
    ParametersModule,
  ],
  controllers: [AppController],
})
export class AppModule {}
