import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { mockJwtAuthGuard } from '../../../test/mock/guards/jwt-auth.mock.guard';
import { mockRoleGuard } from '../../../test/mock/guards/role.mock.guard';
import { CategoriesAccompagnementServiceMock } from '../../../test/mock/services/categoriesAccompagnement.mock.service';
import { JwtAuthGuard } from '../../auth/guards/jwt-auth.guard';
import { RolesGuard } from '../../users/guards/roles.guard';
import { CreateCategoriesAccompagnement } from '../dto/create-categoriesAccompagnement.dto';
import { CategoriesAccompagnement } from '../schemas/categoriesAccompagnement.schema';
import { CategoriesAccompagnementService } from '../services/categories-accompagnement.service';
import { CategoriesAccompagnementController } from './categories-accompagnement.controller';

describe('CategoriesAccompagnementController', () => {
  let controller: CategoriesAccompagnementController;

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [],
      providers: [
        {
          provide: CategoriesAccompagnementService,
          useClass: CategoriesAccompagnementServiceMock,
        },
        {
          provide: getModelToken('CategoriesAccompagnement'),
          useValue: CategoriesAccompagnement,
        },
      ],
      controllers: [CategoriesAccompagnementController],
    })
      .overrideGuard(JwtAuthGuard)
      .useValue(mockJwtAuthGuard)
      .overrideGuard(RolesGuard)
      .useValue(mockRoleGuard)
      .compile();

    controller = module.get<CategoriesAccompagnementController>(CategoriesAccompagnementController);
  });

  it('should be defined', () => {
    expect(controller).toBeDefined();
  });

  it('should find all accompagnements', async () => {
    expect((await controller.findAll()).length).toBe(1);
    expect((await controller.findAll())[0].modules.length).toBe(8);
  });

  it('should create accompagnements', async () => {
    const data: CreateCategoriesAccompagnement = {
      name: 'test',
      modules: [
        {
          id: 'tst',
          text: 'Test',
        },
        {
          id: 'antst',
          text: 'Another test',
        },
      ],
    };
    expect((await controller.create(data)).modules.length).toBe(2);
  });
});
