import { Body, Controller, Post, Get, UseGuards } from '@nestjs/common';
import { ApiTags } from '@nestjs/swagger';
import { JwtAuthGuard } from '../auth/guards/jwt-auth.guard';
import { Roles } from '../users/decorators/roles.decorator';
import { RolesGuard } from '../users/guards/roles.guard';
import { ParametersService } from './parameters.service';
import { Parameters } from './schemas/parameters.schema';

@ApiTags('parameters')
@Controller('parameters')
export class ParametersController {
  constructor(private parametersService: ParametersService) {}

  @Get()
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Roles('admin')
  public async getParameters(): Promise<Parameters> {
    return this.parametersService.getParameters();
  }

  @Post()
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Roles('admin')
  public async setParameterLockdownInfoDisplay(@Body() data: { lockdownInfoDisplay: boolean }): Promise<Parameters> {
    return this.parametersService.setParameterLockdownInfoDisplay(data.lockdownInfoDisplay);
  }
}
