import { Type } from 'class-transformer';
import { ArrayNotEmpty, IsArray, IsNotEmpty, ValidateNested } from 'class-validator';
import { Address } from '../schemas/address.schema';
import { Week } from '../../shared/schemas/week.schema';
import { PersonalOfferDocument } from '../../personal-offers/schemas/personal-offer.schema';

export class StructureDto {
  numero: string;
  createdAt: Date;
  updatedAt: Date;
  deletedAt: Date;

  @IsNotEmpty()
  structureName: string;

  @IsNotEmpty()
  structureType: string;

  description: string;

  @ValidateNested({ each: true })
  @Type(() => Address)
  address: Address;

  contactPhone: string;

  @IsNotEmpty()
  contactMail: string;

  website: string;
  facebook: string;
  twitter: string;
  instagram: string;
  linkedin: string;

  lockdownActivity: string;
  otherDescription: string;
  @IsNotEmpty()
  pmrAccess: boolean;
  publicsAccompaniment: string[];
  proceduresAccompaniment: string[];
  remoteAccompaniment: boolean;
  @ArrayNotEmpty()
  accessModality: string[];

  labelsQualifications: string[];

  @ArrayNotEmpty()
  publics: string[];
  @IsNotEmpty()
  freeWorkShop: boolean | string;
  @IsNotEmpty()
  nbComputers: number;
  @IsNotEmpty()
  nbPrinters: number;
  @IsNotEmpty()
  nbTablets: number;
  @IsNotEmpty()
  nbNumericTerminal: number;
  @IsNotEmpty()
  nbScanners: number;
  exceptionalClosures: string;
  equipmentsAndServices: string[];
  hours: Week;
  baseSkills: string[];
  accessRight: string[];
  parentingHelp: string[];
  socialAndProfessional: string[];
  digitalCultureSecurity: string[];
  coord: number[];
  accountVerified = true; // Value is forced to true because there is no verification needed now. Cannot be removed because of old data prod
  dataShareConsentDate: Date;

  @IsArray()
  personalOffers: PersonalOfferDocument[];
}
